@@ -507,7 +507,7 @@ void wikitotextinline(TCXSTR *rbuf, const char *line){
 
 
 /* Dump the attributes and the body text of an article into an HTML string. */
-void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl){
+void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl, const char *duri){
   assert(rbuf && cols && buri && bhl >= 0);
   char idbuf[NUMBUFSIZ];
   const char *val = tcmapget2(cols, "id");
@@ -549,7 +549,7 @@ void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl){
   const char *text = tcmapget2(cols, "text");
   if(text && *text != '\0'){
     tcxstrprintf(rbuf, "<div class=\"text\">\n");
-    wikitohtml(rbuf, text, *idbuf != '\0' ? idbuf : NULL, buri, bhl + 1);
+    wikitohtml(rbuf, text, *idbuf != '\0' ? idbuf : NULL, buri, bhl + 1, duri);
     tcxstrcat2(rbuf, "</div>\n");
   }
   const char *com = tcmapget2(cols, "comments");
@@ -576,7 +576,7 @@ void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl){
             tcxstrprintf(rbuf, "<span class=\"date\">%@</span> :\n", numbuf);
             tcxstrprintf(rbuf, "<span class=\"owner\">%@</span> :\n", co);
             tcxstrprintf(rbuf, "<span class=\"text\">");
-            wikitohtmlinline(rbuf, ct, buri);
+            wikitohtmlinline(rbuf, ct, buri, duri);
             tcxstrprintf(rbuf, "</span>\n");
             tcxstrprintf(rbuf, "</div>\n");
           }
@@ -591,7 +591,8 @@ void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl){
 
 
 /* Convert a Wiki string into an HTML string. */
-void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl){
+void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl,
+                const char *duri){
   assert(rbuf && str && buri && bhl >= 0);
   TCLIST *lines = tcstrsplit(str, "\n");
   int lnum = tclistnum(lines);
@@ -631,7 +632,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
           tcxstrprintf(rbuf, "\"");
         }
         tcxstrprintf(rbuf, " class=\"ah%d topic\">", abslv);
-        wikitohtmlinline(rbuf, rp, buri);
+        wikitohtmlinline(rbuf, rp, buri, duri);
         tcxstrprintf(rbuf, "</h%d>\n", lv);
         for(int i = lv; i < HEADLVMAX; i++){
           headcnts[lv] = 0;
@@ -670,7 +671,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
           tcxstrprintf(rbuf, "<%s>\n", tag);
         }
         tcxstrcat2(rbuf, "<li>");
-        wikitohtmlinline(rbuf, rp, buri);
+        wikitohtmlinline(rbuf, rp, buri, duri);
       }
       while(tclistnum(stack) > 0){
         tcxstrcat2(rbuf, "</li>\n");
@@ -698,7 +699,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
           const char *pv = strchr(rp, sep);
           char *field = pv ? tcmemdup(rp, pv - rp) : tcstrdup(rp);
           tcstrtrim(field);
-          wikitohtmlinline(rbuf, field, buri);
+          wikitohtmlinline(rbuf, field, buri, duri);
           tcfree(field);
           tcxstrcat2(rbuf, "</td>\n");
           if(!pv) break;
@@ -721,7 +722,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
         rp = tcstrskipspc(rp + 1);
         if(*rp != '\0'){
           tcxstrcat2(rbuf, "<p>");
-          wikitohtmlinline(rbuf, rp, buri);
+          wikitohtmlinline(rbuf, rp, buri, duri);
           tcxstrcat2(rbuf, "</p>\n");
         }
       }
@@ -834,7 +835,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
         char *url;
         if(tcstrfwm(uri, "upfile:")){
           const char *rp = strstr(uri, ":") + 1;
-          url = tcsprintf("%s/%?", buri, rp);
+          url = tcsprintf("%s/%@", duri ? duri : buri, rp);
           name = rp;
         } else {
           url = tcstrdup(uri);
@@ -890,7 +891,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
       line = tcstrskipspc(line);
       if(*line != '\0'){
         tcxstrcat2(rbuf, "<p>");
-        wikitohtmlinline(rbuf, line, buri);
+        wikitohtmlinline(rbuf, line, buri, duri);
         tcxstrcat2(rbuf, "</p>\n");
         ri++;
       } else {
@@ -902,11 +903,8 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
 }
 
 
-/* Add an inline Wiki string into HTML.
-   `rbuf' specifies the result buffer.
-   `line' specifies the inline Wiki string.
-   `buri' specifies the base URI. */
-void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri){
+/* Add an inline Wiki string into HTML. */
+void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const char *duri){
   assert(rbuf && line && buri);
   while(*line != '\0'){
     const char *pv;
@@ -926,63 +924,65 @@ void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri){
           int64_t id = tcatoi(tcstrskipspc(strchr(uri, ':') + 1));
           tcxstrprintf(rbuf, "<a href=\"%s?id=%lld\">", buri, (long long)(id > 0 ? id : 0));
         } else if(tcstrfwm(uri, "name:")){
-          uri  = tcstrskipspc(strchr(uri, ':') + 1);
+          uri = tcstrskipspc(strchr(uri, ':') + 1);
           tcxstrprintf(rbuf, "<a href=\"%s?name=%?\">", buri, uri);
         } else if(tcstrfwm(uri, "param:")){
-          uri  = tcstrskipspc(strchr(uri, ':') + 1);
+          uri = tcstrskipspc(strchr(uri, ':') + 1);
           tcxstrprintf(rbuf, "<a href=\"%s%s%@\">", buri, *uri != 0 ? "?" : "", uri);
         } else if(tcstrfwm(uri, "upfile:")){
-          uri  = tcstrskipspc(strchr(uri, ':') + 1);
-          tcxstrprintf(rbuf, "<a href=\"%s/%@\">", buri, uri);
+          uri = tcstrskipspc(strchr(uri, ':') + 1);
+          tcxstrprintf(rbuf, "<a href=\"%s/%@\">", duri ? duri : buri, uri);
         } else if(tcstrfwm(uri, "wpen:")){
-          uri  = tcstrskipspc(strchr(uri, ':') + 1);
+          uri = tcstrskipspc(strchr(uri, ':') + 1);
+          if(*uri == '\0') uri = field;
           tcxstrprintf(rbuf, "<a href=\"http://en.wikipedia.org/wiki/%?\">", uri);
         } else if(tcstrfwm(uri, "wpja:")){
-          uri  = tcstrskipspc(strchr(uri, ':') + 1);
+          uri = tcstrskipspc(strchr(uri, ':') + 1);
+          if(*uri == '\0') uri = field;
           tcxstrprintf(rbuf, "<a href=\"http://ja.wikipedia.org/wiki/%?\">", uri);
         } else if(sep){
-          uri  = tcstrskipspc(uri);
+          uri = tcstrskipspc(uri);
           tcxstrprintf(rbuf, "<a href=\"%@\">", uri);
         } else {
-          uri  = tcstrskipspc(uri);
+          uri = tcstrskipspc(uri);
           tcxstrprintf(rbuf, "<a href=\"%s?name=%?\">", buri, uri);
         }
-        wikitohtmlinline(rbuf, field, buri);
+        wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrprintf(rbuf, "</a>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[*") && (pv = strstr(line + 2, "*]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<strong>");
-        wikitohtmlinline(rbuf, field, buri);
+        wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</strong>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[\"") && (pv = strstr(line + 2, "\"]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<cite>");
-        wikitohtmlinline(rbuf, field, buri);
+        wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</cite>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[+") && (pv = strstr(line + 2, "+]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<ins>");
-        wikitohtmlinline(rbuf, field, buri);
+        wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</ins>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[-") && (pv = strstr(line + 2, "-]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<del>");
-        wikitohtmlinline(rbuf, field, buri);
+        wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</del>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[#") && (pv = strstr(line + 2, "#]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<code>");
-        wikitohtmlinline(rbuf, field, buri);
+        wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</code>");
         tcfree(field);
         line = pv + 2;
@@ -1065,8 +1065,8 @@ const char *mimetype(const char *name){
     "svg", "image/svg+xml", "xbm", "image/x-xbitmap",
     "au", "audio/basic", "snd", "audio/basic", "mid", "audio/midi", "midi", "audio/midi",
     "mp2", "audio/mpeg", "mp3", "audio/mpeg", "wav", "audio/x-wav", "mpg", "video/mpeg",
-    "mpeg", "video/mpeg", "qt", "video/quicktime", "mov", "video/quicktime",
-    "avi", "video/x-msvideo",
+    "mpeg", "video/mpeg", "mp4", "video/mp4", "qt", "video/quicktime",
+    "mov", "video/quicktime", "avi", "video/x-msvideo",
     NULL
   };
   int len = strlen(name);
@@ -1173,7 +1173,7 @@ char *pathencode(const char *str){
       break;
     } else if(c == ' '){
       *(wp++) = '+';
-    } else if(c < ' ' || c == '/' || c == 0x7f){
+    } else if(c < ' ' || c == '%' || c == '/' || c == 0x7f){
       wp += sprintf(wp, "%%%02X", c);
     } else {
       *(wp++) = c;
