@@ -50,7 +50,7 @@ __COMMON_CLINKAGEBEGIN
  *************************************************************************************************/
 
 
-#define TPVERSION      "0.9.9"
+#define TPVERSION      "0.9.10"
 
 #define TUNEBNUM       131071            // bnum tuning parameter
 #define TUNEAPOW       6                 // apow tuning parameter
@@ -102,8 +102,9 @@ void wikitotextinline(TCXSTR *rbuf, const char *line);
    `rbuf' specifies the result buffer.
    `cols' specifies a map object containing columns.
    `buri' specifies the base URI.
-   `bhl' specifies the base header level. */
-void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl);
+   `bhl' specifies the base header level.
+   `duri' specifie the URI of the data directory. */
+void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl, const char *duri);
 
 
 /* Convert a Wiki string into an HTML string.
@@ -111,15 +112,18 @@ void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl);
    `str' specifies the Wiki string.
    `id' specifies the ID string of the article.  If it is `NULL', the ID is not expressed.
    `buri' specifies the base URI.
-   `bhl' specifies the base header level. */
-void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl);
+   `bhl' specifies the base header level.
+   `duri' specifie the URI of the data directory. */
+void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl,
+                const char *duri);
 
 
 /* Add an inline Wiki string into HTML.
    `rbuf' specifies the result buffer.
    `line' specifies the inline Wiki string.
-   `buri' specifies the base URI. */
-void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri);
+   `buri' specifies the base URI.
+   `duri' specifie the URI of the data directory. */
+void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const char *duri);
 
 
 /* Simplify a date string.
